#ifndef WTRUNTIME_TYPE_H
#define WTRUNTIME_TYPE_H

#include "config.h"
#include "wtruntime.h"
#include "types.h"
#include "byteorder.h"

struct wtr_program;
struct wtr_type;

struct wtr_typeslot
{
	union
	{
		struct wtr_type* type;
		wt_handle        _pad;
	};
};

struct wtr_type
{
	wt_u32              id;       /* global type identifier */
	wt_u32              kind;     /* type kind (WTBSYMT_*) */
	wt_uintptr          refcount; /* reference counter */
	const char*         name;     /* name of the type */
	struct wtr_program* program;  /* owner program */
};

struct wtr_type_field
{
	const char* name;
	wt_u32      type;
};

struct wtr_type_record
{
	struct wtr_type        type;
	wt_u32                 fldc;
	struct wtr_type_field* fldv;
};

struct wtr_type_privaterecord
{
	struct wtr_type type;
};

struct wtr_type_limitedrecord
{
	struct wtr_type type;
};

struct wtr_type_datacons
{
	const char* name;
	wt_u32      typc;
	wt_u32      typv[];
};

struct wtr_type_datatype
{
	struct wtr_type           type;
	wt_u32                    numcons;
	wt_u32                    defcons;
	struct wtr_type_datacons* conses[];
};

struct wtr_type_functype
{
	struct wtr_type type;
	wt_u32          argc;
	wt_u32          retc;
	wt_u32          is_pure;
	wt_u32*         argv;
	wt_u32*         retv;
};

struct wtr_type_enum
{
	struct wtr_type type;
	wt_u32          enumeratorc;
	const char**    enumeratorv;
};

void wtr_type_release(struct wtr_type* type);
wt_bool wtr_type_is_equal(const struct wtr_type* lhs, const struct wtr_type* rhs);
wt_bool wtr_type_is_equal_preserved(const struct wtr_type* lhs, const struct wtr_type* rhs, wt_u32 preserved_rhs_local_typeid);
void wtr_type_post_resolve(struct wtr_type* type, wt_u32 preserved_typeid, wt_u32 resolved_typeid);
struct wtr_type* wtr_type_record_new(struct wtr_program* program, const char* name, wt_u32 fldc, const wt_u32* fldv, wt_bo bo);
struct wtr_type* wtr_type_privaterecord_new(struct wtr_program* program, const char* name);
struct wtr_type* wtr_type_limitedrecord_new(struct wtr_program* program, const char* name);
struct wtr_type* wtr_type_datatype_new(struct wtr_program* program, const char* name, const wt_u32 preserve_typeid, const void* data, wt_bo bo);
struct wtr_type* wtr_type_functype_new(struct wtr_program* program, const char* name, const wt_u32 preserve_typeid, const void* data, wt_bo bo);
struct wtr_type* wtr_type_enum_new(struct wtr_program* program, const char* name, const void* data, wt_bo bo);

#endif /* WTRUNTIME_TYPE_H */
